﻿using BoldReports.Web.ReportViewer;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Caching.Memory;
using Ops.Infrastructure.Server;
using Ops.Infrastructure.Server.Models;
using Ops.Shared.Models.Traceability;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using NSwag.Annotations;
using System.Threading.Tasks;
using static Microsoft.AspNetCore.Http.StatusCodes;
using System.IO;
using System.Collections.Generic;
using System;
using System.Reflection;
using BoldReports.Web;

namespace Ops.Server.Controllers
{
    
    [Route("api/{controller}/{action}/{id?}")]
    public class BoldReportsViewerController : ControllerBase, IReportController, IReportLogger
	{
        // Report viewer requires a memory cache to store the information of consecutive client requests and
        // the rendered report viewer in the server.
        private IMemoryCache _cache;

        // IWebHostEnvironment used with sample to get the application data from wwwroot.
        private IWebHostEnvironment _hostingEnvironment;

        public BoldReportsViewerController(IMemoryCache memoryCache, IWebHostEnvironment hostingEnvironment)
        {
            _cache = memoryCache;
            _hostingEnvironment = hostingEnvironment;
        }

        //Get action for getting resources from the report
        [ActionName("GetResource")]
        [AcceptVerbs("GET")]
        // Method will be called from Report Viewer client to get the image src for Image report item.
        public object GetResource(ReportResource resource)
        {
            return ReportHelper.GetResource(resource, this, _cache);
        }

        // Method will be called to initialize the report information to load the report with ReportHelper for processing.
        
        public void OnInitReportOptions(ReportViewerOptions reportOption)
        {


			reportOption.ReportModel.ReportServerUrl = "http://pc30/ReportServer/";
			reportOption.ReportModel.ReportPath = "/Customers";
			//reportOption.ReportModel.ReportPath = "/SSRS/report/" + reportOption.ReportModel.ReportPath;
			//reportOption.ReportModel.Parameters = parameters;
			//reportOption.ReportModel.ReportServerCredential = new System.Net.NetworkCredential(@"4T\Paul", "CerocR0cks");

			//string basePath = _hostingEnvironment.WebRootPath;
			//    // Here, we have loaded the sales-order-detail.rdl report from the application folder wwwroot\Resources. sales-order-detail.rdl should be in the wwwroot\Resources application folder.
			//    System.IO.FileStream inputStream = new System.IO.FileStream(basePath + @"\resources\" + reportOption.ReportModel.ReportPath + ".rdl", System.IO.FileMode.Open, System.IO.FileAccess.Read);
			//    MemoryStream reportStream = new MemoryStream();
			//    inputStream.CopyTo(reportStream);
			//    reportStream.Position = 0;
			//    inputStream.Close();
			//    reportOption.ReportModel.Stream = reportStream;
			
			// reportOption.ReportModel.DataSourceCredentials.Add(new BoldReports.Web.DataSourceCredentials() { Name = "AdventureWorks", ConnectionString = "Data Source=dataplatformdemodata.syncfusion.com;Initial Catalog=AdventureWorks;", UserId = "demoreadonly@data-platform-demo", Password = "N@c)=Y8s*1&dh" });
		}

		// Method will be called when report is loaded internally to start the layout process with ReportHelper.
		
        public void OnReportLoaded(ReportViewerOptions reportOption)
        {
        }

        [HttpPost]
        public object PostFormReportAction()
        {
            return ReportHelper.ProcessReport(null, this, _cache);
        }

        // Post action to process the report from the server based on json parameters and send the result back to the client.
        [HttpPost]
        public object PostReportAction([FromBody] Dictionary<string, object> jsonArray)
        {
            return ReportHelper.ProcessReport(jsonArray, this, this._cache);
        }

		public void LogError(string message, Exception exception, MethodBase methodType, ErrorType errorType)
		{
			WriteLogs(string.Format("Error Message: {0} \n Stack Trace: {1}", message, exception.StackTrace));
		}

		public void LogError(string errorCode, string message, Exception exception, string errorDetail, string methodName, string className)
		{
			WriteLogs(string.Format("Class Name: {0} \n Method Name: {1} \n Error Message: {2} \n Stack Trace: {3}", className, methodName, errorDetail, exception.StackTrace));
		}

		internal void WriteLogs(string errorMessage)
		{
			// Error details text file path location
			string filePath = Path.Combine(_hostingEnvironment.WebRootPath, "ErrorDetails.txt");
			using (StreamWriter writer = new StreamWriter(filePath, true))
			{
				writer.AutoFlush = true;
				writer.WriteLine(errorMessage);
			}
		}

	}

}